/*
pick4.c  Version 0.3.0  Pick4 Drawings
Copyright (C) 2002-2010   dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Subroutine call:
    	int select;
    	double odds;
    	double getp4(int select);
    	odds = getp4(select);
*/

/* If the chi-square test fails, repeat your test. */
/* The chi-square usually passes in the next test. */
/* If the chi-square fails consistently, then the */
/* program has to be reviewed for correctness. */

/* The higher the number of iterations, the more */
/* likely the chi-square test will pass.  The chi-square */
/* test needs to have its categories filled with at */
/* least a 10 count in order to pass consistently. */
/* With 100000 iterations, the chi-square test passes */
/* consistently. */

/* See the documentation in p4prob.c regarding the */
/* calls to getp4.c. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "pick4.h"
#include "rnd.h"

#define CENTURY 67830
#define FOUR 4
#define TEN 10

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s options\n", pgm);
   fprintf(stderr,"Where options are:\n");
   fprintf(stderr,"   -s is straight match (1/10000)\n");
   fprintf(stderr,"   -b is box match      (24/10000)\n");
   fprintf(stderr,"   -x is straight/box   (1/10000 or 24/10000)\n");
   fprintf(stderr,"   -n 9999 is 4 a digit number 0000-9999\n");
   fprintf(stderr,"   -h is syntax\n");
   fprintf(stderr,"Default is -s -n 1234\n");
   fprintf(stderr,"Example: %s -b -n 8765\n", pgm);
   fprintf(stderr,"To see probabilities, run p4prob\n");
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i,indx;
   int rslt;
   int option;
   int numlen;
   int tktnum;
   int tktdup;
   unsigned char *p,*q;
   unsigned char *gameptr;
   unsigned char *trialptr;
   unsigned char *ticketptr;
   unsigned char ticket[16];
   unsigned char game[16];
   unsigned char trial[16];
   int trialnbr;
   int totmtch,totdup;
   int exactmtch;
   int totplay;
   int maxplay;
   double prob,dblplay;
   double winning;
   double *prizeptr;
   double prize[16];
   double expected;
   double getp4(int select);
   double *bingoptr;
   double bingo[16];
   p = (unsigned char *) ticket;
   q = (unsigned char *) ticket + TEN;
   while (p < q) *p++ = '\0';
   gameptr = (unsigned char *) game;
   option = 0;
   tktnum = 1234;
   gameptr = (unsigned char *) game;
   *gameptr++ = (unsigned char) 1;
   *gameptr++ = (unsigned char) 2;
   *gameptr++ = (unsigned char) 3;
   *gameptr++ = (unsigned char) 4;
   i = 1;
   while (i < argc)
      {
      rslt = strcmp(*(argv+i),"-h");
      if (!rslt) putstx(*argv);
      rslt = strcmp(*(argv+i),"-s");
      if (!rslt)
	 {
	 option = STRAIGHT;
	 i++;
	 continue;
	 } /* if straight */
      rslt = strcmp(*(argv+i),"-b");
      if (!rslt)
	 {
	 option = BOX;
	 i++;
	 continue;
	 } /* if box */
      rslt = strcmp(*(argv+i),"-x");
      if (!rslt)
	 {
	 option = STRBOX;
	 i++;
	 continue;
	 } /* if straight/box */
      rslt = strcmp(*(argv+i),"-n");
      if (!rslt)
	 {
	 i++;
	 numlen = strlen(*(argv+i));
	 if (numlen != 4)
	    {
	    fprintf(stderr,"pick4: invalid "
	       "4 digit number %s\n",
	       *(argv+i));
	    putstx(*argv);
	    } /* invalid ticket number */
	 p = (unsigned char *) *(argv+i);
	 q = (unsigned char *) p + 4;
         gameptr = (unsigned char *) game;
	 while (p < q)
	    {
	    if (*p < '0' || *p > '9')
	       {
	       fprintf(stderr,"pick4: invalid "
	          "4 digit number %s\n",
	          *(argv+i));
	       putstx(*argv);
	       } /* invalid ticket number */
	    indx = (int) *p - '0';
            *gameptr++ = (unsigned char) indx;
            ticketptr = (unsigned char *) ticket + indx;
            (*ticketptr)++;
	    p++;
	    } /* validate each digit */
	 tktnum = atoi(*(argv+i));
	 if (tktnum < 0 || tktnum > 9999)
	    {
	    fprintf(stderr,"pick4: invalid "
	       "4 digit number %s\n",
	       *(argv+i));
	    putstx(*argv);
	    } /* invalid ticket number */
	 i++;
	 continue;
	 } /* if back pair */
      fprintf(stderr,"pick4: invalid option %s\n",
	 *(argv+i));
      putstx(*argv);
      } /* for each program option */
   ticketptr = (unsigned char *) ticket;
   tktdup = 0;
   i = TEN;
   while (i--)
      {
      if (*ticketptr > 1) tktdup += (int) *ticketptr;
      ticketptr++;
      } /* for each digit */
   if (tktdup > 0 && option != STRAIGHT)
      {
      fprintf(stderr,"pick4: "
	 "number %d has duplicates\n",
	 tktnum);
      fprintf(stderr,"Pick 4 odds do "
	 "not favor duplicate numbers,\n");
      fprintf(stderr,"unless you choose "
	 "the straight match option.\n");
      fprintf(stderr,"There are more permutations "
	 "if the pattern is wxyz.\n");
      fprintf(stderr,"This gives you more "
	 "ways of winning Pick 4\n");
      fprintf(stderr,"with the box option "
	 "or the straight/box option.\n");
      putstx(*argv);
      } /* if box or str/bx, too many dups */
   /* 13 times per week for 100 years */
   /* 52 * 100 * 13 = 67600 */
   /* Based on Julian days, it computes to 67830 */
   /* ((2488434.5 - 2451910.5) / 7) * 13 = 67830 */ 
   maxplay = CENTURY;
   sd = (unsigned char *) rndinit();
   prizeptr = (double *) prize;
   *prizeptr++ = 5000.00;
   *prizeptr++ = 200.00;
   *prizeptr++ = 2600.00;
   *prizeptr   = 100.00;
   printf("Pick 4 ticket:  ");
   gameptr = (unsigned char *) game;
   i = FOUR;
   while (i--)
      {
      printf("%2d ", (int) *gameptr++);
      } /* 4 numbers on ticket */
   printf("\n\n# games %d   (13 times "
      "per week, 100 years)\n\n",
      CENTURY);
   bingoptr = (double *) bingo;
   i = 11;
   while (i--) *bingoptr++ = 0.0;
   totplay = maxplay;
   while (totplay--)
      {
      trialptr  = (unsigned char *) trial;
      i = TEN;
      while (i--) *trialptr++ = (unsigned char) '\0';
      trialnbr = exactmtch =
	 totmtch = totdup = 0;
      gameptr = (unsigned char *) game;
      i = 0;
      while (i < FOUR)
         {
         indx = rndnum(TEN,sd);
	 /* trialnbr = (trialnbr * 10) + indx; */
         trialptr  = (unsigned char *) trial  + indx;
	 (*trialptr)++;
	 if (indx == *gameptr) exactmtch++;
	 gameptr++;
	 i++;
         } /* pick 4 balls with replacement */
      ticketptr = (unsigned char *) ticket;
      trialptr  = (unsigned char *) trial;
      i = TEN;
      while (i--)
	 {
	 if (*trialptr == *ticketptr)
	    totmtch += (int) *trialptr;
	 if (*trialptr > 1) totdup += (int) *trialptr;
	 ticketptr++;
	 trialptr++;
	 } /* for each digit */
      if (exactmtch == FOUR)
	 {
         bingoptr = (double *) bingo + STRAIGHT;
         (*bingoptr)++;
         bingoptr = (double *) bingo + BOX;
         (*bingoptr)++;
         bingoptr = (double *) bingo + STRBOXE;
         (*bingoptr)++;
	 } /* if exact 4 */
      else if (totmtch == 4)
	 {
         bingoptr = (double *) bingo + BOX;
         (*bingoptr)++;
         bingoptr = (double *) bingo + STRBOXA;
         (*bingoptr)++;
	 } /* if match in any order */
      } /* tst loop maxplay times */

   bingoptr = (double *) bingo;
   prizeptr = (double *) prize;
   dblplay = (double) maxplay;

   /* straight */
   if (*bingoptr > 0.0)
      {
      winning = ((*prizeptr) * (*bingoptr)) - dblplay;
      } /* if won */
   else winning = 0.0;
   prob = getp4(STRAIGHT);
   expected = dblplay / prob;
   if (option == STRAIGHT)
      {
      printf("Straight Match\n\n");
      printf("Match      Wins      Net Prize    "
         "   Expected           Odds\n");
      printf("Str   %9.0f  %13.2f  %13.5f   %12.1f\n",
         *bingoptr, winning, expected, prob);
      } /* if print */
   bingoptr++;
   prizeptr++;

   /* box */
   if (*bingoptr > 0.0)
      {
      winning = ((*prizeptr) * (*bingoptr)) - dblplay;
      } /* if won */
   else winning = 0.0;
   prob = getp4(BOX);
   expected = dblplay / prob;
   if (option == BOX)
      {
      printf("Box Match\n\n");
      printf("Match      Wins      Net Prize    "
         "   Expected           Odds\n");
      printf("Box   %9.0f  %13.2f  %13.5f   %12.1f\n",
         *bingoptr, winning, expected, prob);
      } /* if print */
   bingoptr++;
   prizeptr++;

   /* straight/box exact order */
   if (*bingoptr > 0.0)
      {
      winning = ((*prizeptr) * (*bingoptr)) - (dblplay * 0.5);
      } /* if won */
   else winning = 0.0;
   prob = getp4(STRAIGHT);
   expected = dblplay / prob;
   if (option == STRBOX)
      {
      printf("Straight/box in exact order\n");
      printf("Match      Wins      Net Prize    "
         "   Expected           Odds\n");
      printf("sbe   %9.0f  %13.2f  %13.5f   %12.1f\n",
         *bingoptr, winning, expected, prob);
      } /* if print */
   bingoptr++;
   prizeptr++;

   /* straight/box any order */
   if (*bingoptr > 0.0)
      {
      winning = ((*prizeptr) * (*bingoptr)) - (dblplay * 0.5);
      } /* if won */
   else winning = 0.0;
   prob = getp4(BOX);
   expected = dblplay / prob;
   if (option == STRBOX)
      {
      printf("\nStraight/box in any order\n");
      printf("Match      Wins      Net Prize    "
         "   Expected           Odds\n");
      printf("sba   %9.0f  %13.2f  %13.5f   %12.1f\n",
         *bingoptr, winning, expected, prob);
      } /* if print */
   bingoptr++;
   prizeptr++;

   free(sd);
   return(0);
   } /* main */
